@@ -1,5 +1,5 @@
 /* DecimalFormat.java -- Formats and parses numbers
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2003 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -37,6 +37,7 @@ exception statement from your version. */
 
 package java.text;
 
+import java.util.Currency;
 import java.util.Locale;
 import java.util.MissingResourceException;
 import java.util.ResourceBundle;
@@ -202,6 +203,8 @@ public class DecimalFormat extends NumberFormat
 	      }
 	    else if (c != syms.getExponential()
 		     && c != syms.getPatternSeparator()
+		     && c != syms.getPercent()
+		     && c != syms.getPerMill()
 		     && patChars.indexOf(c) != -1)
 	      throw new IllegalArgumentException ("unexpected special " +
 						  "character - index: " + index);
@@ -456,7 +459,7 @@ public class DecimalFormat extends NumberFormat
 	    exponent = (long) Math.floor (Math.log(number) / Math.log(10));
 	    if (minimumIntegerDigits > 0)
 	      exponent -= minimumIntegerDigits - 1;
-	    baseNumber = (long) (number / Math.pow(10.0, exponent));
+	    baseNumber = (number / Math.pow(10.0, exponent));
 	  }
 	else
 	  baseNumber = number;
@@ -474,7 +477,7 @@ public class DecimalFormat extends NumberFormat
 	    intPart = Math.floor(intPart / 10);
 
 	    // Append group separator if required.
-	    if (groupingUsed && count > 0 && count % groupingSize == 0)
+	    if (groupingUsed && count > 0 && groupingSize != 0 && count % groupingSize == 0)
 	      dest.insert(index, symbols.getGroupingSeparator());
 
 	    dest.insert(index, (char) (symbols.getZeroDigit() + dig));
@@ -602,7 +605,7 @@ public class DecimalFormat extends NumberFormat
 	  }
 
 	// Append group separator if required.
-	if (groupingUsed && count > 0 && count % groupingSize == 0)
+	if (groupingUsed && count > 0 && groupingSize != 0 && count % groupingSize == 0)
 	  dest.insert(index, symbols.getGroupingSeparator());
 
 	dest.insert(index, (char) (symbols.getZeroDigit() + dig));
@@ -635,6 +638,19 @@ public class DecimalFormat extends NumberFormat
     return dest;
   }
 
+  /**
+   * Returns the currency corresponding to the currency symbol stored
+   * in the instance of <code>DecimalFormatSymbols</code> used by this
+   * <code>DecimalFormat</code>.
+   *
+   * @return A new instance of <code>Currency</code> if
+   * the currency code matches a known one, null otherwise.
+   */
+  public Currency getCurrency()
+  {
+    return symbols.getCurrency();
+  }
+
   public DecimalFormatSymbols getDecimalFormatSymbols ()
   {
     return symbols;
@@ -693,8 +709,8 @@ public class DecimalFormat extends NumberFormat
     int index = pos.getIndex();
     StringBuffer buf = new StringBuffer ();
 
-      // We have to check both prefixes, because one might be empty.
-      // We want to pick the longest prefix that matches.
+    // We have to check both prefixes, because one might be empty.  We
+    // want to pick the longest prefix that matches.
     boolean got_pos = str.startsWith(positivePrefix, index);
     String np = (negativePrefix != null
 		 ? negativePrefix
@@ -729,11 +745,14 @@ public class DecimalFormat extends NumberFormat
 
     // FIXME: handle Inf and NaN.
 
-      // FIXME: do we have to respect minimum/maxmimum digit stuff?
-      // What about leading zeros?  What about multiplier?
+    // FIXME: do we have to respect minimum digits?
+    // What about leading zeros?  What about multiplier?
 
     int start_index = index;
     int max = str.length();
+    int last = index + maximumIntegerDigits;
+    if (last > 0 && max > last)
+      max = last;
     char zero = symbols.getZeroDigit();
     int last_group = -1;
     boolean int_part = true;
@@ -745,7 +764,8 @@ public class DecimalFormat extends NumberFormat
 	// FIXME: what about grouping size?
 	if (groupingUsed && c == symbols.getGroupingSeparator())
 	  {
-	    if (last_group != -1
+	    if (last_group != -1 
+		&& groupingSize != 0  
 		&& (index - last_group) % groupingSize != 0)
 	      {
 		pos.setErrorIndex(index);
@@ -762,7 +782,8 @@ public class DecimalFormat extends NumberFormat
 	  break;
 	else if (c == symbols.getDecimalSeparator())
 	  {
-	    if (last_group != -1
+	    if (last_group != -1 
+		&& groupingSize != 0 
 		&& (index - last_group) % groupingSize != 0)
 	      {
 		pos.setErrorIndex(index);
@@ -849,6 +870,16 @@ public class DecimalFormat extends NumberFormat
     return result;
   }
 
+  /**
+   * Sets the <code>Currency</code> on the
+   * <code>DecimalFormatSymbols</code> used, which also sets the
+   * currency symbols on those symbols.
+   */
+  public void setCurrency(Currency currency)
+  {
+    symbols.setCurrency(currency);
+  }
+
   public void setDecimalFormatSymbols (DecimalFormatSymbols newSymbols)
   {
     symbols = newSymbols;
@@ -866,22 +897,22 @@ public class DecimalFormat extends NumberFormat
 
   public void setMaximumFractionDigits (int newValue)
   {
-    maximumFractionDigits = Math.min(newValue, 340);
+    super.setMaximumFractionDigits(Math.min(newValue, 340));
   }
 
   public void setMaximumIntegerDigits (int newValue)
   {
-    maximumIntegerDigits = Math.min(newValue, 309);
+    super.setMaximumIntegerDigits(Math.min(newValue, 309));
   }
 
   public void setMinimumFractionDigits (int newValue)
   {
-    minimumFractionDigits = Math.min(newValue, 340);
+    super.setMinimumFractionDigits(Math.min(newValue, 340));
   }
 
   public void setMinimumIntegerDigits (int newValue)
   {
-    minimumIntegerDigits = Math.min(newValue, 309);
+    super.setMinimumIntegerDigits(Math.min(newValue, 309));
   }
 
   public void setMultiplier (int newValue)
